/**
  * Color.fx
  *
  * Author:        Peter B. Tummillo
  * Last Modified: 5/8/12
  *
  * Description:   Renders an object with a color determined by its vertices.
  */

/////////////////////////////////////////////////////////////////////
// Constant Buffers
/////////////////////////////////////////////////////////////////////

cbuffer cbPerFrame
{
	float4x4 gVP;
};

cbuffer cbPerObject
{
	float4x4 gWorld;
};


/////////////////////////////////////////////////////////////////////
// Global Vars
/////////////////////////////////////////////////////////////////////

Texture2D gDiffuseMap;


/////////////////////////////////////////////////////////////////////
// State
/////////////////////////////////////////////////////////////////////

SamplerState gDefaultSampler
{
    Filter   = MIN_MAG_MIP_LINEAR;
    AddressU = CLAMP;
    AddressV = CLAMP;
};


/////////////////////////////////////////////////////////////////////
// Shader Input Structures
/////////////////////////////////////////////////////////////////////

struct VS_IN
{
	float3 posL   : POSITION;
	float4 color  : COLOR;
};

struct VS_IN_TEX
{
	float3 posL   : POSITION;
	float2 texC   : TEXCOORD;
	float4 color  : COLOR;
};

struct VS_OUT
{
	float4 posH  : SV_POSITION;
	float4 color : COLOR;
};

struct VS_OUT_TEX
{
	float4 posH  : SV_POSITION;
	float2 texC  : TEXCOORD;
	float4 color : COLOR;
};


/////////////////////////////////////////////////////////////////////
// Vertex Shaders
/////////////////////////////////////////////////////////////////////

VS_OUT VS(VS_IN input)
{
	VS_OUT output;

	// Transform vertex position to homogeneous clip space
	float4x4 WVP = mul( gWorld, gVP );
	output.posH = mul( float4( input.posL, 1.0f ), WVP );
	
	// Just pass vertex color into the pixel shader
    output.color = input.color;
	
	return output;
}

VS_OUT_TEX VS_Textured(VS_IN_TEX input)
{
	VS_OUT_TEX output;

	// Transform vertex position to homogeneous clip space
	float4x4 WVP = mul( gWorld, gVP );
	output.posH = mul( float4( input.posL, 1.0f ), WVP );
	
	// Just pass vertex color and texture coordinate into the pixel shader
    output.texC  = input.texC;
	output.color = input.color;
	
	return output;
}


/////////////////////////////////////////////////////////////////////
// Pixel Shaders
/////////////////////////////////////////////////////////////////////

float4 PS(VS_OUT input) : SV_Target
{
	// Return default color
	return input.color;
}

float4 PS_Textured(VS_OUT_TEX input) : SV_Target
{
	// Sample texture
	float4 diffuse = gDiffuseMap.Sample( gDefaultSampler, input.texC );

	// Mix texture with color
	return input.color * diffuse;
}


/////////////////////////////////////////////////////////////////////
// Techniques
/////////////////////////////////////////////////////////////////////

technique10 Color
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}

technique10 TexturedColor
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS_Textured() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS_Textured() ) );
	}
}
