/**
  * Lighting.fx
  *
  * Author:        Peter B. Tummillo
  * Last Modified: 5/17/12
  *
  * Description:   Renders an object with a color determined by its material properties.
  */

/////////////////////////////////////////////////////////////////////
// Constant Buffers
/////////////////////////////////////////////////////////////////////

cbuffer cbPerFrame
{
	float4x4 gVP;

	float3 gCameraPosition = float3( 0.0f, 0.0f, 0.0f );

	float4 gLightColor = float4( 1.0f, 1.0f, 1.0f, 1.0f );
	float3 gLightPosition;
	//float3 gLightDir = float3( 0.0f, 1.0f, 0.0f );
};

cbuffer cbPerObject
{
	float4x4 gWorld;
	
	float4 gAmbient;
	float4 gDiffuse;
	float4 gSpecular;

	int gShininess;
	float gAlpha;
};


/////////////////////////////////////////////////////////////////////
// Global Vars
/////////////////////////////////////////////////////////////////////

Texture2D gDiffuseMap;


/////////////////////////////////////////////////////////////////////
// State
/////////////////////////////////////////////////////////////////////

SamplerState gDefaultSampler
{
    Filter   = MIN_MAG_MIP_LINEAR;
    AddressU = CLAMP;
    AddressV = CLAMP;
};


/////////////////////////////////////////////////////////////////////
// Shader Input Structures
/////////////////////////////////////////////////////////////////////

struct VS_IN
{
    float3 posL    : POSITION;
    float3 normalL : NORMAL;
    float2 texC    : TEXCOORD;
};

struct VS_OUT
{
    float4 posH  : SV_POSITION;
    float4 color : COLOR0;
    float2 texC  : TEXCOORD;
};


/////////////////////////////////////////////////////////////////////
// Shaders
/////////////////////////////////////////////////////////////////////

VS_OUT VS( VS_IN input, uniform bool bSpecular )
{
    VS_OUT output;
    
    // Transform vertex position into world space for lighting,
	// and homogenous clip space for display
	float4x4 WVP = mul( gWorld, gVP );
    float3 posW = mul( input.posL, (float3x3)gWorld );
    output.posH = mul( float4( input.posL, 1.0f ), WVP );

    // Transform the normal into world space for lighting
    float3 normalW = mul( input.normalL, (float3x3)gWorld );

    // Pass the texture coordinate
    output.texC = input.texC;

    // Compute the light vector
    float3 lightDir = normalize( posW.xyz - gLightPosition );

    // Compute the ambient and diffuse components of illumination
    output.color.rgb = gLightColor * gAmbient;
    output.color.rgb += gLightColor * gDiffuse * saturate( dot( lightDir, normalW ) );

    // If enabled, calculate the specular term
    if ( bSpecular )
    {
        float3 viewVector = normalize( posW.xyz - gCameraPosition );
        float3 reflectionVector = reflect( lightDir, normalW );
        float  phoneValue = saturate( dot( reflectionVector, viewVector ) );

        output.color.rgb += gSpecular * pow( phoneValue, gShininess );
    }

    output.color.a = gAlpha;

    return output;
}

float4 PS( VS_OUT input, uniform bool bTexture ) : SV_Target
{
    float4 output = input.color;

    // Sample and modulate the texture
    if ( bTexture )
    {
        output *= gDiffuseMap.Sample( gDefaultSampler, input.texC );
    }

    return output;
}


/////////////////////////////////////////////////////////////////////
// Techniques
/////////////////////////////////////////////////////////////////////

technique10 Lambert
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS(false) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS(false) ) );
    }
}

technique10 Phong
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS(true) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS(false) ) );
    }
}

technique10 TexturedLambert
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS(false) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS(true) ) );
    }
}

technique10 TexturedPhong
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS(true) ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS(true) ) );
    }
}
