/**
  * SpriteBatch.fx
  *
  * Author:         Peter B. Tummillo
  * Contributor(s): Devin Ford
  * Last Modified:  2/23/12
  * 
  * Description:    Shader for rendering sprites with textures.
  */

cbuffer cbPerFrame
{
	float4x4 gWVP; 
};

cbuffer cbRarely
{
	uint gSamplerFilter;
	bool gIsTextureClamped;
};


// Sprite Texture
Texture2D gDiffuseMap;


// Clamp
SamplerState g2DSamplerClamp
{
	Filter   = gSamplerFilter;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

// Wrap
SamplerState g2DSamplerWrap
{
	Filter   = gSamplerFilter;
	AddressU = WRAP;
	AddressV = WRAP;
};


struct VS_IN
{
	float3 posL  : POSITION;
	float2 texC  : TEXCOORD;
	float4 color : COLOR;
};

struct VS_OUT
{
	float4 posH  : SV_POSITION;
	float2 texC  : TEXCOORD;
	float4 color : COLOR;
};


/*
 * Vertex Shader
 *
 * Summary: Transforms vertices.
 */
VS_OUT VS(VS_IN input)
{
	VS_OUT output;

	// Transform vertex position to homogeneous clip space
	output.posH = mul( float4(input.posL, 1.0f), gWVP );
	
	// Just pass color and texture coordinate into the pixel shader
	output.texC  = input.texC;
	output.color = input.color;

	return output;
}

/*
 * Pixel Shader
 *
 * Summary: Applies textures.
 */
float4 PS(VS_OUT input) : SV_Target
{
	// Sample the texture
	float4 diffuse = float4(0.0f, 0.0f, 0.0f, 0.0f);
	if (gIsTextureClamped)
		diffuse = gDiffuseMap.Sample( g2DSamplerClamp, input.texC );
	else
		diffuse = gDiffuseMap.Sample( g2DSamplerWrap,  input.texC );

	// Mix texture with color
	return input.color * diffuse;
}


technique10 SpriteBatchTech
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
